import os
from contextlib import contextmanager
from pathlib import Path
from typing import Dict, Iterator


@contextmanager
def dir_changer(path: Path) -> Iterator[None]:
    """A context manager that changes the current working directory"""
    old_cwd = Path.cwd()
    os.chdir(path)
    try:
        yield
    finally:
        os.chdir(old_cwd)


@contextmanager
def env_changer(env: Dict[str, str]) -> Iterator[None]:
    """A context manager that changes the current working directory"""
    old_env = os.environ.copy()
    os.environ.update(env)
    try:
        yield
    finally:
        os.environ.clear()
        os.environ.update(old_env)



import os
from pathlib import Path
from zipfile import ZipFile

from addon_odoo_wheel.builder import build_editable
from tests.utils import dir_changer


def test_build_editable(project0: Path, tmp_path: Path) -> None:
    with dir_changer(project0):
        wheel_name = build_editable(os.fspath(tmp_path))
        # test build directory content
        build_dir = project0 / "build"
        assert build_dir.is_dir()
        assert build_dir.joinpath(".gitignore").is_file()
        editable_dir = build_dir / "__editable__"
        editable_manifest_path = editable_dir / "odoo" / "addons" / "project0" / "__manifest__.py"
        assert editable_manifest_path.is_file()
        # test wheel contenet
        assert (tmp_path / wheel_name).exists()
        with ZipFile(tmp_path / wheel_name) as zf:
            names = zf.namelist()
            assert "odoo/addons/project0/__manifest__.py" not in names
            assert "addon-odoo-project0.pth" in names
            assert zf.open("addon-odoo-project0.pth", "r").read().decode("utf-8") == str(editable_dir.resolve())



from __future__ import annotations

from abc import ABC, abstractmethod
from pathlib import Path
from typing import Callable

from importlib_metadata import entry_points
from manifestoo_core.git_postversion import POST_VERSION_STRATEGY_DOT_N
from manifestoo_core.metadata import OdooSeriesInfo
from manifestoo_core.odoo_series import OdooSeries

__all__ = ["ContributorABC", "OdooSeries"]


class ContributorABC(ABC):
    names = []

    @abstractmethod
    def get_mail(self, odoo_series: OdooSeries | None) -> str | None:
        """
        Allow you to define an email based on the odoo series if you want.
        The odoo series is passed as a parameter only if it can be defined. See the documentation.
        :param odoo_series: Allow to change the mail depending of the version if you want
        :return: A mail or None if you don't want to define an email
        """

    def odoo_series_required(self) -> bool:
        """
        Allow to tell if you want to build an addon if the version of Odoo can't be defined.
        :return: True, mean the build will failed if the odoo series can't be defined
        """
        return False

    @abstractmethod
    def get_package_prefix(self, odoo_series: OdooSeries | None) -> str | None:
        """
        The prefix of the wheel name.
        The final wheel name will be the following: <prefix>-<addon_name>.whl
        :param odoo_series: Allow to change the name depending of the version if you want
        :return: The prefix, if none is returned, only the addon name will be used
        """

    def git_postversion_strategy(self, odoo_series: OdooSeries | None) -> str:
        if odoo_series:
            return OdooSeriesInfo.from_odoo_series(odoo_series).git_postversion_strategy
        return POST_VERSION_STRATEGY_DOT_N

    def get_classifiers(self, odoo_series: OdooSeries | None) -> list[str]:
        """
        Allow to add classifiers to the metadata
        See https://pypi.org/classifiers/ to search for available classifiers
        :param odoo_series: Allow to change the classifiers depending of the version if you want
        :return:
        """
        default = [
            "Programming Language :: Python",
            "Framework :: Odoo",
        ]
        if odoo_series:
            default.append(f"Framework :: Odoo :: {odoo_series.value}")
        return default

    def pkg_exclude(self, dist: str) -> Callable[[str, list[str]], list[str]]:
        def _callback_ignore(src, names):
            if Path(src).name == "static":
                return ["description"]
            return []

        return _callback_ignore


class ContributorFactory:
    names = []
    _registry_well_know: dict[str | None, ContributorABC] = {}

    @classmethod
    def _load_plugins(cls):
        if cls._registry_well_know:
            return
        odoo_addon_contributor = entry_points(group="addon_odoo_wheel.contributor")
        for ep in odoo_addon_contributor:
            klass = ep.load()
            for name in klass.names:
                if name in cls._registry_well_know:
                    raise ValueError(f"Addon contributor {name} already registered")
                cls._registry_well_know[name and name.upper()] = klass()

    @classmethod
    def from_author(cls, name: str | None) -> ContributorABC:
        cls._load_plugins()
        return cls._registry_well_know.get(name and name.upper() or None) or _UnknownAddonContributor()


class _UnknownAddonContributor(ContributorABC):
    names = [None, ""]

    def get_mail(self, odoo_serie_info: OdooSeries | None) -> str | None:
        return None

    def get_package_prefix(self, odoo_series: OdooSeries | None) -> str:
        return "addon-odoo"



from pathlib import Path

from build import ProjectBuilder
from packaging.metadata import Metadata
from packaging.requirements import Requirement
from packaging.version import Version


def test_project0(module_data: Path, tmp_path: Path) -> None:
    builder = ProjectBuilder(module_data / "project0")
    metadata_dir = builder.metadata_path(tmp_path)
    metadata_path = Path(metadata_dir) / "METADATA"
    assert metadata_path.is_file()
    metadata = Metadata.from_email(metadata_path.read_bytes())
    assert metadata.name == "addon-odoo-project0", "Lib name if empty is auto computed with 'addon-odoo-' prefix"
    assert set(metadata.classifiers) == {
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python",
        "Framework :: Odoo",
        "Framework :: Odoo :: 18.0",
        "License :: OSI Approved :: GNU Affero General Public License v3",
    }
    assert metadata.version == Version("18.0.1.2.3")
    assert metadata.license_expression == "AGPL-3.0"
    assert metadata.project_urls["Homepage"] == "https://website.fr"
    assert metadata.author == "Author"
    assert not metadata.author_email
    assert metadata.requires_dist == [Requirement("requests>=2.32")]


def test_project1(module_data: Path, tmp_path: Path) -> None:
    builder = ProjectBuilder(module_data / "project1")
    metadata_dir = builder.metadata_path(tmp_path)
    metadata_path = Path(metadata_dir) / "METADATA"
    assert metadata_path.is_file()
    metadata = Metadata.from_email(metadata_path.read_bytes())
    assert metadata.name == "addon-odoo-project1", "Lib name if empty is auto computed with 'addon-odoo-' prefix"
    assert set(metadata.requires_dist) == {
        Requirement("requests>=2.32"),
    }
    assert set(metadata.classifiers) == {
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python",
        "Framework :: Odoo",
        "Framework :: Odoo :: 18.0",
        "License :: OSI Approved :: GNU Affero General Public License v3",
    }
    assert metadata.version == Version("18.0.1.2.3")
    assert metadata.license_expression == "AGPL-3.0"
    assert metadata.project_urls["Homepage"] == "https://website.fr"


def test_project2(module_data: Path, tmp_path: Path) -> None:
    builder = ProjectBuilder(module_data / "project2")
    metadata_dir = builder.metadata_path(tmp_path)
    metadata_path = Path(metadata_dir) / "METADATA"
    assert metadata_path.is_file()
    metadata = Metadata.from_email(metadata_path.read_bytes())
    dependencies = set(metadata.requires_dist)
    assert dependencies == {
        Requirement("requests>=2.32"),
        Requirement('odoo-addon-queue-job>=18.0; extra == "queue"'),
        Requirement('pytest; extra == "dev"'),
        Requirement('pytest-cov; extra == "dev"'),
    }
    assert metadata.name == "company-addon-project"
    assert set(metadata.classifiers) == {
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python",
        "Framework :: Odoo",
        "Framework :: Odoo :: 18.0",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Topic :: Software Development :: Build Tools",
    }
    assert metadata.version == Version("18.0.1.2.3")
    assert metadata.license_expression == "AGPL-3.0"
    assert metadata.project_urls["Homepage"] == "https://mangono.fr"



"""
This module provides functionality to build wheels and source distributions (sdists)
for Odoo addons, following PEP 517 and PEP 660.

- [PEP 517](https://peps.python.org/pep-0517/): A standard for specifying build backends.
- [PEP 660](https://peps.python.org/pep-0660/): A standard for creating editable wheels.
"""

from __future__ import annotations

import logging
import os
import shutil
import tarfile
import tempfile
from pathlib import Path
from typing import Any

from wheel.wheelfile import WheelFile

from . import utils
from .metadata import AddonMetadata

logging.basicConfig(level=logging.WARNING)
_logger = logging.getLogger(__name__)


class AddonBuilder:
    def __init__(self, distribution_addon_dir: Path, config: dict):
        force_addon_name = None
        if (distribution_addon_dir / "odoo-addon-name.txt").exists():
            with (distribution_addon_dir / "odoo-addon-name.txt").open("r") as f:
                force_addon_name = f.read()
        self.addon_metadata = AddonMetadata.from_addon_dir(
            distribution_addon_dir, allow_not_installable=True, force_addon_name=force_addon_name
        )
        self.config = config
        self.addon_dir = distribution_addon_dir

    def build_sdist_to(self, sdist_directory: Path) -> str:
        sdist_tar_name = self.addon_metadata.sdist_name + ".tar.gz"
        with tempfile.TemporaryDirectory() as tmpdir:
            sdist_tmpdir = Path(tmpdir) / self.addon_metadata.sdist_name
            utils.copy_to(
                self.addon_metadata.build_path,
                sdist_tmpdir,
                pkg_exclude=self.addon_metadata.module_contributor.pkg_exclude("sdist"),
            )
            utils.write_metadata(self.addon_metadata.metadata, dest=sdist_tmpdir / "PKG-INFO")
            (sdist_tmpdir / "odoo-addon-name.txt").write_text(self.addon_metadata.addon_name)
            with tarfile.open(
                str(sdist_directory / sdist_tar_name),
                mode="w|gz",
                format=tarfile.PAX_FORMAT,
            ) as tf:
                tf.add(str(sdist_tmpdir), arcname=self.addon_metadata.sdist_name)
        return sdist_tar_name

    def make_dist_info(self, dst: Path) -> str:
        dist_info_dirname = f"{self.addon_metadata.sdist_name}.dist-info"
        dist_info_path = dst / dist_info_dirname
        dist_info_path.mkdir()
        utils.write_metadata(utils.base_wheel_metadata(), dest=dist_info_path / "WHEEL")
        utils.write_metadata(self.addon_metadata.metadata, dest=dist_info_path / "METADATA")
        (dist_info_path / "top_level.txt").write_text("odoo")
        entry_points = self.addon_metadata.pyproject_options.entry_points
        if not entry_points:
            entry_points = self.addon_metadata.pyproject_options.addon_odoo_wheel.entry_points
        utils.create_entry_point(
            entry_points, python_addon_path=self.addon_metadata.python_addon_path, dest=dist_info_path
        )
        return dist_info_dirname

    def build_wheel_to(self, wheel_directory: Path, *, editable: bool = False) -> str:
        with tempfile.TemporaryDirectory() as tmpdir:
            tmppath = Path(tmpdir)
            # always include metadata
            self.make_dist_info(tmppath)
            if editable:
                # Prepare {addon_dir}/build/__editable__/odoo/addon/{addon_name} symlink
                self._make_wheel_editable(tmppath)
            else:
                self._make_static_wheel(tmppath)

            wheel_path = os.path.join(wheel_directory, self.addon_metadata.wheel_name + ".whl")
            with WheelFile(wheel_path, "w") as wf:
                _logger.info(f"Repacking wheel as {wheel_path}...")
                wf.write_files(str(tmppath.absolute()))
        return self.addon_metadata.wheel_name + ".whl"

    def _make_static_wheel(self, tmppath):
        odoo_addon_path = tmppath / "odoo" / "addons"
        odoo_addon_path.mkdir(parents=True)
        odoo_addon_path = odoo_addon_path / self.addon_metadata.addon_name
        utils.copy_to(
            self.addon_metadata.addon_path,
            odoo_addon_path,
            pkg_exclude=self.addon_metadata.module_contributor.pkg_exclude("whl"),
        )
        # we don't want pyproject.toml nor PKG-INFO in the wheel
        utils.ensure_absent(
            [
                odoo_addon_path / "pyproject.toml",
                odoo_addon_path / "PKG-INFO",
                odoo_addon_path / "odoo-addon-name.txt",
                odoo_addon_path / "requirements.txt",
            ]
        )

    def _make_wheel_editable(self, tmppath: Path):
        build_dir = self.addon_metadata.addon_path / "build"
        build_dir.mkdir(parents=True, exist_ok=True)
        build_dir.joinpath(".gitignore").write_text("*")
        editable_dir = build_dir / "__editable__"
        if editable_dir.is_dir():
            shutil.rmtree(editable_dir)
        editable_addons_dir = editable_dir / "odoo" / "addons"
        editable_addons_dir.mkdir(parents=True, exist_ok=True)
        editable_addon_symlink = editable_addons_dir / self.addon_metadata.addon_name
        editable_addon_symlink.symlink_to(self.addon_metadata.addon_path, target_is_directory=True)
        # Add .pth file pointing to {addon_dir}/build/__editable__ into the wheel
        tmppath.joinpath(self.addon_metadata.metadata["Name"] + ".pth").write_text(str(editable_dir.resolve()))


def build_wheel(
    wheel_directory: str, config_settings: dict[str, Any] | None = None, metadata_directory: str | None = None
) -> str:
    """
    Build a wheel for the Odoo addon.
    Following PEP 517

    This function creates a non-editable wheel for the Odoo addon present in the
    current working directory. It utilizes the AddonWheelBuilder to perform the
    build process.

    Args:
        wheel_directory (str): The directory where the built wheel will be stored.
        config_settings (Optional[Dict[str, Any]]): Optional configuration settings
            for the build process. Defaults to None.
        metadata_directory (Optional[str]): Optional directory for metadata.
            Defaults to None.

    Returns:
        str: The name of the built wheel file.
    """
    return AddonBuilder(Path.cwd(), config_settings).build_wheel_to(Path(wheel_directory), editable=False)


def build_editable(
    wheel_directory: str,
    config_settings: dict[str, Any] | None = None,
    metadata_directory: str | None = None,
) -> str:
    """
    Build an editable wheel for the Odoo addon.

    Following PEP 660 (editable wheels)

    This function creates an editable wheel for the Odoo addon present in the
    current working directory. It utilizes the AddonWheelBuilder to perform the
    build process.

    Args:
        wheel_directory (str): The directory where the built wheel will be stored.
        config_settings (Optional[Dict[str, Any]]): Optional configuration settings
            for the build process. Defaults to None.
        metadata_directory (Optional[str]): Optional directory for metadata.
            Defaults to None.

    Returns:
        str: The name of the built wheel file.
    """
    return AddonBuilder(Path.cwd(), config_settings).build_wheel_to(Path(wheel_directory), editable=True)


def prepare_metadata_for_build_wheel(metadata_directory: str, config_settings: dict[str, Any] | None = None) -> str:
    """
    Prepare the metadata for a wheel build.

    Following PEP 517

    This function creates the necessary metadata for a wheel build of the Odoo addon
    present in the current working directory. It utilizes the package_metadata_from_addon
    function to create the metadata and the _make_dist_info function to write the metadata
    to disk.

    Args:
        metadata_directory (str): The directory where the metadata will be stored.
        config_settings (Optional[Dict[str, Any]]): Optional configuration settings
            for the build process. Defaults to None.

    Returns:
        str: The name of the .dist-info directory created.
    """
    return AddonBuilder(Path.cwd(), config_settings).make_dist_info(Path(metadata_directory))


prepare_metadata_for_build_editable = prepare_metadata_for_build_wheel


def build_sdist(sdist_directory: str, config_settings: dict[str, Any] | None = None) -> str:
    """
    Build a source distribution (sdist) for the Odoo addon.

    Following PEP 517

    This function creates an sdist for the Odoo addon present in the
    current working directory. It utilizes the _build_sdist function to perform the
    build process.

    Args:
        sdist_directory (str): The directory where the built sdist will be stored.
        config_settings (Optional[Dict[str, Any]]): Optional configuration settings
            for the build process. Defaults to None.

    Returns:
        str: The name of the built sdist file.
    """
    return AddonBuilder(Path.cwd(), config_settings).build_sdist_to(Path(sdist_directory))



class UnsupportedOperation(NotImplementedError):
    pass


class WhoolMangonoException(Exception):
    pass


class NoScmFound(WhoolMangonoException):
    pass


class OnlyCommunityAddonsException(WhoolMangonoException):
    pass


class CantResolveAddonException(WhoolMangonoException):
    pass



from pathlib import Path

import pytest
from manifestoo_core.addon import Addon
from manifestoo_core.exceptions import UnsupportedManifestVersion
from manifestoo_core.manifest import Manifest
from manifestoo_core.odoo_series import OdooSeries
from packaging.requirements import Requirement

from addon_odoo_wheel import metadata
from addon_odoo_wheel.utils import PyProjectConfig


def test_dynamic_version(module_data: Path):
    addon_metadata = metadata.AddonMetadata(
        module_data,
        PyProjectConfig.from_config(
            module_data,
            {
                "project": {
                    "name": "company-addon-name",
                    "dynamic": ["version"],
                },
                "tool": {"addon-odoo-wheel": {"metadata_src": "project"}},
            },
        ),
        Addon(
            name="addon_name",
            manifest=Manifest.from_dict(
                dict(
                    version="18.0.1.2.3",
                    depends=["base"],
                    installable=True,
                )
            ),
            manifest_path=module_data,
        ),
    )
    assert addon_metadata.addon.name == "addon_name"
    assert addon_metadata.metadata["Version"] == addon_metadata.addon.manifest.version == "18.0.1.2.3"
    assert addon_metadata.odoo_series == OdooSeries.v18_0
    assert addon_metadata.metadata["Name"] == "company-addon-name"
    assert addon_metadata.wheel_name == "company_addon_name-18.0.1.2.3-py3-none-any"
    assert addon_metadata.sdist_name == "company_addon_name-18.0.1.2.3"
    assert not addon_metadata.metadata["Requires-Dist"]


def test_not_same_version(module_data: Path):
    with pytest.raises(ValueError) as excinfo:
        metadata.AddonMetadata(
            module_data,
            PyProjectConfig.from_config(
                module_data,
                {
                    "project": {
                        "name": "company-addon-name",
                        "version": "2.3.4",
                    }
                },
            ),
            Addon(
                name="addon_name",
                manifest=Manifest.from_dict(
                    dict(
                        version="18.0.1.2.3",
                        depends=["base"],
                        installable=True,
                    )
                ),
                manifest_path=module_data,
            ),
        )
    assert excinfo.value.args[0] == (
        "Manifest version or pyproject.toml version must be the same, or set version in pyproject.toml to dynamic"
    )


def test_fixed_version(module_data: Path):
    addon_metadata = metadata.AddonMetadata(
        module_data,
        PyProjectConfig.from_config(
            module_data,
            {
                "project": {
                    "name": "company-addon-name",
                    "version": "18.0.1.2.3",
                },
                "tool": {"addon-odoo-wheel": {"metadata_src": "project"}},
            },
        ),
        Addon(
            name="addon_name",
            manifest=Manifest.from_dict(
                dict(
                    version="18.0.1.2.3",
                    depends=["base"],
                    installable=True,
                )
            ),
            manifest_path=module_data,
        ),
    )
    assert addon_metadata.addon.name == "addon_name"
    assert addon_metadata.addon.manifest.version == addon_metadata.metadata["Version"] == "18.0.1.2.3"
    assert addon_metadata.odoo_series == OdooSeries.v18_0
    assert addon_metadata.metadata["Name"] == "company-addon-name"
    assert addon_metadata.wheel_name == "company_addon_name-18.0.1.2.3-py3-none-any"
    assert addon_metadata.sdist_name == "company_addon_name-18.0.1.2.3"
    assert not addon_metadata.metadata["Requires-Dist"]


def test_depends_manifest(module_data: Path):
    addon_metadata = metadata.AddonMetadata(
        module_data,
        PyProjectConfig.from_config(
            module_data,
            {
                "project": {
                    "name": "company-addon-name",
                    "dynamic": ["version"],
                    "dependencies": ["odoo-addon-web-environment-ribbon>=18.0"],
                },
                "tool": {"addon-odoo-wheel": {"metadata_src": "project"}},
            },
        ),
        Addon(
            name="addon_name",
            manifest=Manifest.from_dict(
                dict(
                    version="18.0.1.2.3",
                    depends=["base", "purchase", "web_environment_ribbon"],
                    installable=True,
                )
            ),
            manifest_path=module_data,
        ),
    )
    assert addon_metadata.addon.name == "addon_name"
    assert addon_metadata.odoo_series == OdooSeries.v18_0
    assert addon_metadata.metadata["Requires-Dist"] == str(Requirement("odoo-addon-web-environment-ribbon>=18.0"))


def test_force_odoo_series_version(module_data: Path):
    addon_metadata = metadata.AddonMetadata(
        module_data,
        PyProjectConfig.from_config(
            module_data,
            {
                "project": {
                    "name": "company-addon-name",
                },
                "tool": {"addon-odoo-wheel": {"odoo_version_override": "18.0", "metadata_src": "project"}},
            },
        ),
        Addon(
            name="addon_name",
            manifest=Manifest.from_dict(
                dict(
                    version="1.2.3",
                    depends=["base"],
                    installable=True,
                )
            ),
            manifest_path=module_data,
        ),
    )
    assert addon_metadata.addon.name == "addon_name"
    assert addon_metadata.addon.manifest.version == "1.2.3"
    assert addon_metadata.metadata["Version"] == "18.0.1.2.3"
    assert addon_metadata.odoo_series == OdooSeries.v18_0
    assert addon_metadata.metadata["Name"] == "company-addon-name"
    assert addon_metadata.wheel_name == "company_addon_name-18.0.1.2.3-py3-none-any"
    assert addon_metadata.sdist_name == "company_addon_name-18.0.1.2.3"
    assert not addon_metadata.metadata["Requires-Dist"]


def test_auto_config_1(module_data: Path):
    """
    Test auto defining name off the wheel from the name of the odoo module.
    The version is defined from the manifest too.
    """
    module1_path = module_data / "module_with_version"

    addon_metadata = metadata.AddonMetadata(
        module_data,
        PyProjectConfig.from_config(module_data, {}),
        Addon(
            name="my_odoo_addon",
            manifest=Manifest.from_dict(
                dict(
                    version="18.0.1.2.3",
                    depends=["base"],
                    installable=True,
                )
            ),
            manifest_path=module1_path,
        ),
    )
    assert addon_metadata.addon.name == "my_odoo_addon"
    assert addon_metadata.metadata["Version"] == addon_metadata.addon.manifest.version == "18.0.1.2.3"
    assert addon_metadata.odoo_series == OdooSeries.v18_0
    assert addon_metadata.wheel_name == "addon_odoo_my_odoo_addon-18.0.1.2.3-py3-none-any"
    assert addon_metadata.sdist_name == "addon_odoo_my_odoo_addon-18.0.1.2.3"
    assert not addon_metadata.metadata["Requires-Dist"]


def test_force_semver_1(module_data: Path):
    addon_metadata = metadata.AddonMetadata(
        module_data,
        PyProjectConfig.from_config(
            module_data,
            {
                "tool": {"addon-odoo-wheel": {"odoo_version_override": "18.0"}},
            },
        ),
        Addon(
            name="addon_name",
            manifest=Manifest.from_dict(
                dict(
                    version="18.0",
                    depends=["base"],
                    installable=True,
                )
            ),
            manifest_path=module_data,
        ),
    )
    assert addon_metadata.addon.name == "addon_name"
    assert addon_metadata.addon.manifest.version == "18.0"
    assert addon_metadata.metadata["Version"] == "18.0.1.0.0"


def test_force_semver_2(module_data: Path):
    addon_metadata = metadata.AddonMetadata(
        module_data,
        PyProjectConfig.from_config(
            module_data,
            {
                "tool": {"addon-odoo-wheel": {"odoo_version_override": "18.0"}},
            },
        ),
        Addon(
            name="addon_name",
            manifest=Manifest.from_dict(
                dict(
                    version="2",
                    depends=["base"],
                    installable=True,
                )
            ),
            manifest_path=module_data,
        ),
    )
    assert addon_metadata.addon.name == "addon_name"
    assert addon_metadata.addon.manifest.version == "2"
    assert addon_metadata.metadata["Version"] == "18.0.2.0.0"


def test_force_semver_3(module_data: Path):
    addon_metadata = metadata.AddonMetadata(
        module_data,
        PyProjectConfig.from_config(
            module_data,
            {
                "tool": {"addon-odoo-wheel": {"odoo_version_override": "18.0"}},
            },
        ),
        Addon(
            name="addon_name",
            manifest=Manifest.from_dict(
                dict(
                    version="2.1",
                    depends=["base"],
                    installable=True,
                )
            ),
            manifest_path=module_data,
        ),
    )
    assert addon_metadata.addon.name == "addon_name"
    assert addon_metadata.addon.manifest.version == "2.1"
    assert addon_metadata.metadata["Version"] == "18.0.2.1.0"


def test_force_semver_4(module_data: Path):
    addon_metadata = metadata.AddonMetadata(
        module_data,
        PyProjectConfig.from_config(
            module_data,
            {
                "tool": {"addon-odoo-wheel": {"odoo_version_override": "18.0"}},
            },
        ),
        Addon(
            name="addon_name",
            manifest=Manifest.from_dict(
                dict(
                    version="2.1.3",
                    depends=["base"],
                    installable=True,
                )
            ),
            manifest_path=module_data,
        ),
    )
    assert addon_metadata.addon.name == "addon_name"
    assert addon_metadata.addon.manifest.version == "2.1.3"
    assert addon_metadata.metadata["Version"] == "18.0.2.1.3"


def test_force_semver_5(module_data: Path):
    with pytest.raises(UnsupportedManifestVersion) as excinfo:
        metadata.AddonMetadata(
            module_data,
            PyProjectConfig.from_config(
                module_data,
                {
                    "tool": {"addon-odoo-wheel": {"odoo_version_override": "18.0"}},
                },
            ),
            Addon(
                name="addon_name",
                manifest=Manifest.from_dict(
                    dict(
                        version="16.0.2.1.3",
                        depends=["base"],
                        installable=True,
                    )
                ),
                manifest_path=module_data,
            ),
        ).generate_metadata()
    assert excinfo.value.args[0] == ("The version in __manifest__.py start with 16.0 but 18.0 is found")



import os
from pathlib import Path
from tarfile import TarFile

from addon_odoo_wheel.builder import AddonBuilder, build_sdist
from tests.utils import dir_changer


def test_build_sdist(project0: Path, tmp_path: Path) -> None:
    with dir_changer(project0):
        sdist_name = build_sdist(os.fspath(tmp_path))
        assert sdist_name == "addon_odoo_project0-18.0.1.2.3.tar.gz"
        assert (tmp_path / sdist_name).exists()


def test_build_sdist_p3(project3: Path, tmp_path: Path) -> None:
    with dir_changer(project3):
        sdist_name = build_sdist(os.fspath(tmp_path))
        assert sdist_name == "mangono_addon_module_odoo-18.0.1.2.3.tar.gz"
        assert (tmp_path / sdist_name).exists()
        with TarFile.open(tmp_path / sdist_name, mode="r:gz") as tf1:
            tf1_names = sorted(tf1.getnames())
            tf1.extractall(tmp_path)
        assert "mangono_addon_module_odoo-18.0.1.2.3/PKG-INFO" in tf1_names
        assert "mangono_addon_module_odoo-18.0.1.2.3/pyproject.toml" in tf1_names


def test_build_sdist_from_sdist(addon1_sdist_builder: AddonBuilder, tmp_path: Path) -> None:
    assert addon1_sdist_builder.addon_metadata.sdist_name == "company_addon_project-18.0.1.2.3"
    sdist_name = addon1_sdist_builder.build_sdist_to(tmp_path)
    assert sdist_name == "company_addon_project-18.0.1.2.3.tar.gz"
    # extract sdist and test that the root directory has the correct name

    tmp_path2 = tmp_path / "2"
    tmp_path2.mkdir()
    with TarFile.open(tmp_path / sdist_name, mode="r:gz") as tf1:
        tf1_names = sorted(tf1.getnames())
        tf1.extractall(tmp_path2)
    assert "company_addon_project-18.0.1.2.3/PKG-INFO" in tf1_names
    assert "company_addon_project-18.0.1.2.3/pyproject.toml" in tf1_names
    # build sdist from sdist
    tmp_path3 = tmp_path / "3"
    tmp_path3.mkdir()
    sdist_name = AddonBuilder(
        tmp_path2 / "company_addon_project-18.0.1.2.3", addon1_sdist_builder.config
    ).build_sdist_to(tmp_path3)
    assert sdist_name == "company_addon_project-18.0.1.2.3.tar.gz"
    # extract 2nd sdist and test that the root directory has the correct name
    with TarFile.open(tmp_path3 / sdist_name, mode="r:gz") as tf2:
        tf2_names = sorted(tf2.getnames())
        tf2.extractall(tmp_path3)
    # content of both sdists must be identical
    assert tf1_names == tf2_names
    # PKG-INFO in both sdists must be identical
    assert (tmp_path2 / "company_addon_project-18.0.1.2.3" / "PKG-INFO").read_bytes() == (
        tmp_path3 / "company_addon_project-18.0.1.2.3" / "PKG-INFO"
    ).read_bytes()



from __future__ import annotations

from typing import Callable

from manifestoo_core.metadata import OdooSeriesInfo
from manifestoo_core.odoo_series import OdooSeries

from .contributors_api import ContributorABC


class OcaContributor(ContributorABC):
    names = ["OCA", "Odoo Community Association (OCA)"]

    def get_mail(self, odoo_serie_info: OdooSeries | None) -> str:
        return "support@odoo-community.org"

    def get_package_prefix(self, odoo_series: OdooSeries | None) -> str:
        return odoo_series and OdooSeriesInfo.from_odoo_series(odoo_series).pkg_name_pfx or "odoo-addon"


class MangonoContributor(ContributorABC):
    names = ["Mangono", "NDP Systemes", "NDP Systèmes"]

    def get_mail(self, odoo_serie: OdooSeries | None) -> str:
        return "opensource+odoo@mangono.fr"

    def get_package_prefix(self, odoo_serie: OdooSeries | None) -> str:
        return "mangono-addon"


class OdooContributor(ContributorABC):
    names = ["Odoo", "Odoo SA"]

    def get_mail(self, odoo_serie: OdooSeries | None) -> str:
        return "support@odoo.com"

    def get_package_prefix(self, odoo_serie: OdooSeries | None) -> str:
        return "odoo-addon-official"

    def pkg_exclude(self, dist: str) -> Callable[[str, list[str]], list[str]]:
        return lambda src, names: []



from __future__ import annotations
from addon_odoo_wheel.contributors_api import ContributorABC, OdooSeries


class SampleCompanyContributor(ContributorABC):
    names = ["SAMPLE_COMPANY"]  # <.>

    def get_mail(self, odoo_series: OdooSeries | None) -> str:
        return f"odoo+{odoo_series and odoo_series.value or 'any'}@sample_company.com"  # <.>

    def odoo_series_required(self) -> bool:
        return True  # <.>

    def get_package_prefix(self, odoo_series: OdooSeries | None) -> str | None:
        return "sample-company-addon"  # <.>

    def git_postversion_strategy(self, odoo_series: OdooSeries | None) -> str:
        return super().git_postversion_strategy(odoo_series)  # <.>

    def get_classifiers(self, odoo_series: OdooSeries | None) -> list[str]:  # <.>
        return super().get_classifiers(odoo_series) + [
            "Private :: Do Not Upload",
        ]



import os
from pathlib import Path

from manifestoo_core.addons_set import AddonsSet


def main() -> None:
    addon_set = AddonsSet()
    for path in os.listdir(str(Path.cwd())):
        addon_set.add_from_addons_dir(Path(path))

    for addon_name, addon in addon_set.items():
        print(f"=== {addon_name} ===")
        print(f"  - version: {addon.manifest.version}")
        print(f"  - description: {addon.manifest.description}")
        print("Build Source distribution")


if __name__ == "__main__":
    main()
